@@ -52,7 +52,7 @@ pow_int <- function(a, b) a^round(b) # b should be integer.  rounding it makes f
 #'
 #' \code{nimRep} is NIMBLE's version of \code{rep}.  It should behave identically to \code{rep}.  There are no NIMBLE versions of \code{rep.int} or \code{rep_len}.
 #'
-#' \code{nimSeq} is NIMBLE's version of \code{seq}.  It behaves like \code{seq} with support for \code{from}, \code{to}, \code{by} and \code{length.out} arguments.  The \code{along.with} argument is not supported.  There are no NIMBLE versions of \code{seq.int}, \code{seq_along} or \code{seq_len}, with the exception that \code{seq_along} can take a nimbleFunctionList as an argument to provide the index range of a for-loop (\href{https://r-nimble.org/html_manual/cha-welcome-nimble.html}{User Manual} Ch. 13). 
+#' \code{nimSeq} is NIMBLE's version of \code{seq}.  It behaves like \code{seq} with support for \code{from}, \code{to}, \code{by} and \code{length.out} arguments.  The \code{along.with} argument is not supported.  There are no NIMBLE versions of \code{seq.int}, \code{seq_along} or \code{seq_len}, with the exception that \code{seq_along} can take a nimbleFunctionList as an argument to provide the index range of a for-loop (\href{https://r-nimble.org/manual/cha-welcome-nimble.html}{User Manual} Ch. 13). 
 #'
 #' \code{which} behaves like the R version but without support for \code{arr.ind} or \code{useNames} arguments.
 #'
@@ -843,7 +843,7 @@ nimCopy <- function(from, to, nodes = NULL, nodesTo = NULL, row = NA, rowTo = NA
 #'
 #' For internal access to methods of \code{nf}, see \code{\link{nfMethod}}.
 #' 
-#' For more information, see \code{?nimbleFunction} and the NIMBLE \href{https://r-nimble.org/html_manual/cha-welcome-nimble.html}{User Manual}.
+#' For more information, see \code{?nimbleFunction} and the NIMBLE \href{https://r-nimble.org/manual/cha-welcome-nimble.html}{User Manual}.
 #'
 #' @return whatever varName is in the nimbleFunction nf.
 #' @examples
@@ -1497,15 +1497,15 @@ nimOptimDefaultControl <- function() {
   control <- optimControlNimbleList$new()
   control$trace <- 0
   control$fnscale <- 1
-  control$parscale <- NA # Must be filled in to length of par
-  control$ndeps <- NA    # Ditto
-  control$maxit <- NA  ## The default value depends on method.
+  control$parscale <- as.numeric(NA) # Must be filled in to length of par
+  control$ndeps <- as.numeric(NA)    # Ditto
+  control$maxit <- as.integer(NA)  ## The default value depends on method.
   control$abstol = -Inf
   control$reltol <- sqrt(.Machine$double.eps)
   control$alpha <- 1.0
   control$beta <- 0.5
   control$gamma <- 2.0
-  control$REPORT <- NA # Method dependent and not used in compiled version
+  control$REPORT <- as.integer(NA) # Method dependent and not used in compiled version
   control$type <- 1
   control$lmm <- 5
   control$factr <- 1e7
