@@ -33,13 +33,17 @@ ndf_createDetermSimulate <- function(LHS, RHS, dynamicIndexLimitsExpr, RHSnonRep
 
 ## changes 'dnorm(mean=1, sd=2)' into 'rnorm(1, mean=1, sd=2)'
 ndf_createStochSimulate <- function(LHS, RHS, dynamicIndexLimitsExpr, RHSnonReplaced, nodeDim) {
-    BUGSdistName <- as.character(RHS[[1]])
-    RHS[[1]] <- as.name(getDistributionInfo(BUGSdistName)$simulateName)   # does the appropriate substituion of the distribution name
+    subdone <- FALSE
+    BUGSdistName <- safeDeparse(RHS[[1]])
+    distInfo <- getDistributionInfo(BUGSdistName)
+    sim_code <- distInfo$sim_code
+    if(is.null(sim_code)) stop("Could not find simulation ('r') function for ", BUGSdistName)
+    RHS[[1]] <- sim_code
     if(length(RHS) > 1) {    for(i in (length(RHS)+1):3)   { RHS[i] <- RHS[i-1];     names(RHS)[i] <- names(RHS)[i-1] } }    # scoots all named arguments right 1 position
     RHS[[2]] <- 1;     names(RHS)[2] <- ''    # adds the first (unnamed) argument '1'    
     if("lower_" %in% names(RHS) || "upper_" %in% names(RHS)) {
-        RHS <- ndf_createStochSimulateTrunc(RHS, discrete = getAllDistributionsInfo('discrete')[BUGSdistName])
-    } 
+        RHS <- ndf_createStochSimulateTrunc(RHS, distInfo = distInfo, discrete = getAllDistributionsInfo('discrete')[BUGSdistName])
+    }
     if(nimbleOptions()$allowDynamicIndexing && !is.null(dynamicIndexLimitsExpr)) {
         if(is.null(nodeDim)) {
             nanExpr <- NaN
@@ -65,27 +69,32 @@ ndf_createStochSimulate <- function(LHS, RHS, dynamicIndexLimitsExpr, RHSnonRepl
 
 ## changes 'rnorm(mean=1, sd=2, lower_=0, upper_=3)' into correct truncated simulation
 ##   using inverse CDF
-ndf_createStochSimulateTrunc <- function(RHS, discrete = FALSE) {
+ndf_createStochSimulateTrunc <- function(RHS, distInfo, discrete = FALSE) {
     lowerPosn <- which("lower_" == names(RHS))
     upperPosn <- which("upper_" == names(RHS))
     lower <- RHS[[lowerPosn]]
     upper <- RHS[[upperPosn]]
     RHS <- RHS[-c(lowerPosn, upperPosn)]
-    dist <- substring(as.character(RHS[[1]]), 2, 1000)
-    
+    d_code <- distInfo$density_code
+
+    cdf_code <- distInfo$cdf_code
+    quantile_code <- distInfo$quantile_code
+    if(is.null(cdf_code) || is.null(quantile_code))
+      stop("Could not find probability ('p') and/or quantile ('q') function for ", distInfo$BUGSdistName)
+
     lowerTailName <- 'lower.tail' 
     logpName <- 'log.p' 
     logName <- 'log' 
     # setup for runif(1, pdist(lower,...), pdist(upper,...))
     # pdist() expression template for inputs to runif()
     pdistTemplate <- RHS
-    pdistTemplate[[1]] <- as.name(paste0("p", dist))
+    pdistTemplate[[1]] <- cdf_code # as.name(paste0("p", dist))
     pdistTemplate <- addArg(pdistTemplate, 1, lowerTailName)
     pdistTemplate <- addArg(pdistTemplate, 0, logpName)
 
     if(discrete) {
         ddistTemplate <- RHS
-        ddistTemplate[[1]] <- as.name(paste0("d", dist))
+        ddistTemplate[[1]] <- d_code # as.name(paste0("d", dist))
         ddistTemplate <- addArg(ddistTemplate, 0, logName)
         ceilTemplate <- quote(ceiling(x))
     } else ddistTemplate <- NULL
@@ -121,7 +130,7 @@ ndf_createStochSimulateTrunc <- function(RHS, discrete = FALSE) {
         VALUE = VALUE_EXPR)), list(e = substCode)))
 
     # create full qdist(runif(...),...) expression
-    RHS[[1]] <- as.name(paste0("q", dist))
+    RHS[[1]] <- quantile_code # as.name(paste0("q", dist))
     RHS[[2]] <- RUNIF_EXPR
     RHS <- addArg(RHS, 1, lowerTailName)
     RHS <- addArg(RHS, 0, logpName)
@@ -133,12 +142,13 @@ ndf_createStochSimulateTrunc <- function(RHS, discrete = FALSE) {
 ndf_createStochCalculate <- function(logProbNodeExpr, LHS, RHS, diff = FALSE,
                                      ##ADFunc = FALSE,
                                      dynamicIndexLimitsExpr, RHSnonReplaced) {
-    BUGSdistName <- as.character(RHS[[1]])
-    RHS[[1]] <- as.name(getDistributionInfo(BUGSdistName)$densityName)   # does the appropriate substituion of the distribution name
+    BUGSdistName <- safeDeparse(RHS[[1]])
+    distInfo <- getDistributionInfo(BUGSdistName)
+    RHS[[1]] <- distInfo$density_code
     if(length(RHS) > 1) {    for(i in (length(RHS)+1):3)   { RHS[i] <- RHS[i-1];     names(RHS)[i] <- names(RHS)[i-1] } }    # scoots all named arguments right 1 position
     RHS[[2]] <- LHS;     names(RHS)[2] <- ''    # adds the first (unnamed) argument LHS
     if("lower_" %in% names(RHS) || "upper_" %in% names(RHS)) {
-        return(ndf_createStochCalculateTrunc(logProbNodeExpr, LHS, RHS, diff = diff, discrete = getAllDistributionsInfo('discrete')[BUGSdistName], dynamicIndexLimitsExpr = dynamicIndexLimitsExpr, RHSnonReplaced = RHSnonReplaced))
+        return(ndf_createStochCalculateTrunc(logProbNodeExpr, LHS, RHS, diff = diff, distInfo=distInfo, discrete = getAllDistributionsInfo('discrete')[BUGSdistName], dynamicIndexLimitsExpr = dynamicIndexLimitsExpr, RHSnonReplaced = RHSnonReplaced))
     } else {
         userDist <- BUGSdistName %in% getAllDistributionsInfo('namesVector', userOnly = TRUE)
         RHS <- addArg(RHS, 1, 'log')  # adds the last argument log=TRUE (log_value for user-defined) # This was changed to 1 from TRUE for easier C++ generation
@@ -180,25 +190,31 @@ ndf_createStochCalculate <- function(logProbNodeExpr, LHS, RHS, diff = FALSE,
 }
 
 ## changes 'dnorm(mean=1, sd=2, lower=0, upper=3)' into correct truncated calculation
-ndf_createStochCalculateTrunc <- function(logProbNodeExpr, LHS, RHS, diff = FALSE, discrete = FALSE, dynamicIndexLimitsExpr, RHSnonReplaced) {
+ndf_createStochCalculateTrunc <- function(logProbNodeExpr, LHS, RHS, diff = FALSE, distInfo, discrete = FALSE, dynamicIndexLimitsExpr, RHSnonReplaced) {
     lowerPosn <- which("lower_" == names(RHS))
     upperPosn <- which("upper_" == names(RHS))
     lower <- RHS[[lowerPosn]]
     upper <- RHS[[upperPosn]]
     RHS <- RHS[-c(lowerPosn, upperPosn)]
-    dist <- substring(as.character(RHS[[1]]), 2, 1000)
+    d_code <- distInfo$density_code
+#    dist <- safeDeparse(RHS[[1]])
+
     lowerTailName <- 'lower.tail' 
     logpName <- 'log.p' 
     logName <- 'log' 
 
+    cdf_code <- distInfo$cdf_code
+    if(is.null(cdf_code))
+      stop("Could not find probability ('p') function for ", distInfo$BUGSdistName)
+
     pdistTemplate <- RHS
-    pdistTemplate[[1]] <- as.name(paste0("p", dist))
+    pdistTemplate[[1]] <- cdf_code # as.name(paste0("p", dist))
     pdistTemplate <- addArg(pdistTemplate, 1, lowerTailName)
     pdistTemplate <- addArg(pdistTemplate, 0, logpName)
 
     if(discrete) {
         ddistTemplate <- RHS
-        ddistTemplate[[1]] <- as.name(paste0("d", dist))
+        ddistTemplate[[1]] <- d_code # as.name(paste0("d", dist))
         ddistTemplate <- addArg(ddistTemplate, 0, logName)
         ceilTemplate <- quote(ceiling(x))
     } else ddistTemplate <- NULL
@@ -366,36 +382,39 @@ ndf_createSingleMethod <- function(type, nDim) {
     eval(methodDef)
 }
 
-ndf_createVirtualNodeFunctionDefinition <- function(types = list()) {
-    methodsList <- lapply(types, function(singleType) ndf_createSingleMethod(type=singleType$type, nDim=singleType$nDim))
-    if(length(methodsList) > 0)     names(methodsList) <- paste0('get_', names(methodsList))
-    virtualFunctionDef <- substitute(
-        nimbleFunctionVirtual(
-            contains = 'nodeFun',
-            methods = METHODS
-        ),
-        list(METHODS = methodsList)
-    )
-    return(virtualFunctionDef)
-}
-
-ndf_createVirtualNodeFunctionDefinitionsList <- function(userAdded = FALSE) {
-    defsList <- list()
-    if(!userAdded) {
-        defsList$node_determ <- ndf_createVirtualNodeFunctionDefinition()
-        for(distName in getAllDistributionsInfo('namesVector', nimbleOnly = TRUE)) {
-            defsList[[paste0('node_stoch_', distName)]] <- ndf_createVirtualNodeFunctionDefinition(getDistributionInfo(distName)$types)
-        }
-    } else {
-        # this deals with user-provided distributions
-        if(exists('distributions', nimbleUserNamespace, inherits = FALSE)) {
-            for(distName in getAllDistributionsInfo('namesVector', userOnly = TRUE))
-                defsList[[paste0('node_stoch_', distName)]] <- ndf_createVirtualNodeFunctionDefinition(getDistributionInfo(distName)$types)
-        } else stop("ndf_createVirtualNodeFunctionDefinitionsList: no 'distributions' list in nimbleUserNamespace.")
+if(getNimbleOption('enableVirtualNodeFunctionDefs')) {  ## NCT issue 500. Deprecating and will remove in next release.
+    
+    ndf_createVirtualNodeFunctionDefinition <- function(types = list()) {
+        methodsList <- lapply(types, function(singleType) ndf_createSingleMethod(type=singleType$type, nDim=singleType$nDim))
+        if(length(methodsList) > 0)     names(methodsList) <- paste0('get_', names(methodsList))
+        virtualFunctionDef <- substitute(
+            nimbleFunctionVirtual(
+                contains = 'nodeFun',
+                methods = METHODS
+            ),
+            list(METHODS = methodsList)
+        )
+        return(virtualFunctionDef)
     }
-    return(defsList)
-}
 
-virtualNodeFunctionDefinitions <- ndf_createVirtualNodeFunctionDefinitionsList()
-createNamedObjectsFromList(virtualNodeFunctionDefinitions)
+    ndf_createVirtualNodeFunctionDefinitionsList <- function(userAdded = FALSE) {
+        defsList <- list()
+        if(!userAdded) {
+            defsList$node_determ <- ndf_createVirtualNodeFunctionDefinition()
+            for(distName in getAllDistributionsInfo('namesVector', nimbleOnly = TRUE)) {
+                defsList[[paste0('node_stoch_', distName)]] <- ndf_createVirtualNodeFunctionDefinition(getDistributionInfo(distName)$types)
+            }
+        } else {
+                                        # this deals with user-provided distributions
+            if(exists('distributions', nimbleUserNamespace, inherits = FALSE)) {
+                for(distName in getAllDistributionsInfo('namesVector', userOnly = TRUE))
+                    defsList[[paste0('node_stoch_', distName)]] <- ndf_createVirtualNodeFunctionDefinition(getDistributionInfo(distName)$types)
+            } else stop("ndf_createVirtualNodeFunctionDefinitionsList: no 'distributions' list in nimbleUserNamespace.")
+        }
+        return(defsList)
+    }
 
+    virtualNodeFunctionDefinitions <- ndf_createVirtualNodeFunctionDefinitionsList()
+    createNamedObjectsFromList(virtualNodeFunctionDefinitions)
+
+} else ndf_createVirtualNodeFunctionDefinitionsList <- function(userAdded = FALSE) {}  # Needed for R CMD check checking of variables.
